"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Code = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _react = _interopRequireDefault(require("react"));

var _clsx = _interopRequireDefault(require("clsx"));

var _uiSchema = require("@ui-schema/ui-schema");

var _reactUid = require("react-uid");

var _FormLabel = _interopRequireDefault(require("@material-ui/core/FormLabel"));

var _makeStyles = _interopRequireDefault(require("@material-ui/core/styles/makeStyles"));

var _LocaleHelperText = require("@ui-schema/ds-material/Component/LocaleHelperText");

var _reactCodemirror = require("react-codemirror2");

var _CodeProvider = require("./CodeProvider");

var useStyles = (0, _makeStyles["default"])({
  root: {
    flex: '1 1 auto',
    marginTop: 0,
    height: '100%',
    position: 'relative',
    '& .CodeMirror': {
      position: 'absolute',
      top: 0,
      bottom: 0,
      left: 0,
      right: 0,
      height: '100%'
    }
  }
});

var Code = function Code(_ref) {
  var storeKeys = _ref.storeKeys,
      ownKey = _ref.ownKey,
      schema = _ref.schema,
      value = _ref.value,
      onChange = _ref.onChange,
      showValidity = _ref.showValidity,
      valid = _ref.valid,
      errors = _ref.errors,
      required = _ref.required;
  var uid = (0, _reactUid.useUID)();

  var _useWidgetCode = (0, _CodeProvider.useWidgetCode)(),
      theme = _useWidgetCode.theme;

  var _React$useState = _react["default"].useState(1),
      _React$useState2 = (0, _slicedToArray2["default"])(_React$useState, 2),
      lines = _React$useState2[0],
      setLines = _React$useState2[1];

  var _React$useState3 = _react["default"].useState(),
      _React$useState4 = (0, _slicedToArray2["default"])(_React$useState3, 2),
      editor = _React$useState4[0],
      setEditor = _React$useState4[1];

  var _React$useState5 = _react["default"].useState(false),
      _React$useState6 = (0, _slicedToArray2["default"])(_React$useState5, 2),
      focused = _React$useState6[0],
      setFocused = _React$useState6[1];

  var classes = useStyles();
  var format = schema.get('format'); // todo: multi format w/ selection

  if (typeof format !== 'string') return null;

  _react["default"].useEffect(function () {
    onChange((0, _uiSchema.updateValidity)(storeKeys, valid));
  }, [valid]);

  _react["default"].useEffect(function () {
    if (editor && editor.refresh) {
      editor.refresh();
    }
  }, [editor]);

  var height = 0;

  if (editor && editor.display && editor.display.lineDiv && editor.display.lineDiv.children && editor.display.lineDiv.children[0]) {
    height = editor.display.lineDiv.children[0].clientHeight;
  }

  if (editor && editor.display && editor.display.gutters && editor.display.wrapper) {
    if (schema.getIn(['view', 'bg']) !== true) {
      editor.display.gutters.style.background = 'transparent';
      editor.display.wrapper.style.background = 'transparent';
    }
  }

  var handleOnChange = _react["default"].useCallback(function (editor) {
    if (editor.doc.size !== lines) {
      setLines(editor.doc.size);
    }
  }, [setLines, lines]);

  var handleBeforeChange = _react["default"].useCallback(function (editor, data, value) {
    onChange((0, _uiSchema.updateValue)(storeKeys, value));
  }, [onChange, storeKeys]);

  var options = _react["default"].useMemo(function () {
    return {
      mode: format,
      lineNumbers: true
    };
  }, [format]);

  if (theme) {
    options.theme = theme;
  }

  return /*#__PURE__*/_react["default"].createElement(_react["default"].Fragment, null, /*#__PURE__*/_react["default"].createElement(_FormLabel["default"], {
    error: !valid && showValidity,
    style: {
      marginBottom: 12,
      display: 'block'
    }
  }, (0, _uiSchema.beautifyKey)(ownKey, schema.get('tt')), required ? ' *' : null), /*#__PURE__*/_react["default"].createElement("div", {
    style: {
      minHeight: '2rem',
      height: lines ? (lines + 1) * height + 6 : 'auto',
      position: 'relative'
    }
  }, /*#__PURE__*/_react["default"].createElement(_reactCodemirror.Controlled, {
    className: (0, _clsx["default"])(classes.root, 'uis-' + uid, 'MuiInput-underline', focused ? 'Mui-focused' : null),
    editorDidMount: setEditor,
    value: value,
    onBeforeChange: handleBeforeChange,
    onChange: handleOnChange,
    onFocus: function onFocus() {
      return setFocused(true);
    },
    onBlur: function onBlur() {
      return setFocused(false);
    },
    options: options
  })), /*#__PURE__*/_react["default"].createElement(_LocaleHelperText.ValidityHelperText, {
    errors: errors,
    showValidity: showValidity,
    schema: schema
  }));
};

exports.Code = Code;